import pickle
import cv2
import mediapipe as mp
import numpy as np

# Charger le modèle
try:
    model_dict = pickle.load(open('./model.p', 'rb'))
    model = model_dict['model']
except FileNotFoundError:
    print("Erreur : Le fichier 'model.p' est introuvable.")
    exit()
except Exception as e:
    print(f"Erreur lors du chargement du modèle : {e}")
    exit()

# Initialiser la capture vidéo
cap = cv2.VideoCapture(0)
if not cap.isOpened():
    print("Erreur : Impossible d'ouvrir la webcam.")
    exit()

# Initialiser Mediapipe
mp_hands = mp.solutions.hands
mp_drawing = mp.solutions.drawing_utils
mp_drawing_styles = mp.solutions.drawing_styles
hands = mp_hands.Hands(static_image_mode=False, min_detection_confidence=0.3, min_tracking_confidence=0.3)

labels_dict = {0: 'O', 1: 'X', 2: 'Y'}
num_features_expected = 84  # On s'assure que l'entrée a toujours 84 caractéristiques

while True:
    data_aux = []
    x_ = []
    y_ = []

    ret, frame = cap.read()
    if not ret:
        print("Erreur : Impossible de capturer une image depuis la webcam.")
        break

    H, W, _ = frame.shape
    frame_rgb = cv2.cvtColor(frame, cv2.COLOR_BGR2RGB)
    results = hands.process(frame_rgb)

    if results.multi_hand_landmarks:
        for hand_landmarks in results.multi_hand_landmarks:
            mp_drawing.draw_landmarks(
                frame, hand_landmarks, mp_hands.HAND_CONNECTIONS,
                mp_drawing_styles.get_default_hand_landmarks_style(),
                mp_drawing_styles.get_default_hand_connections_style()
            )

            # Extraction des coordonnées des 21 landmarks de la main
            for landmark in hand_landmarks.landmark:
                x_.append(landmark.x)
                y_.append(landmark.y)

            # Normalisation des coordonnées
            for landmark in hand_landmarks.landmark:
                data_aux.append(landmark.x - min(x_))
                data_aux.append(landmark.y - min(y_))

        # 🔹 Correction : Duplication des valeurs si on a moins de 84 caractéristiques
        while len(data_aux) < num_features_expected:
            data_aux.extend(data_aux[:num_features_expected - len(data_aux)])  # Duplique les valeurs existantes

        # 🔹 Vérification finale avant prédiction
        if len(data_aux) > num_features_expected:
            data_aux = data_aux[:num_features_expected]  # Tronquer si on dépasse 84

        prediction = model.predict([np.asarray(data_aux)])[0]
        predicted_character = labels_dict.get(int(prediction), "?")

        # Déterminer les coordonnées de la boîte englobante
        x1 = int(min(x_) * W) - 10
        y1 = int(min(y_) * H) - 10
        x2 = int(max(x_) * W) + 10
        y2 = int(max(y_) * H) + 10

        cv2.rectangle(frame, (x1, y1), (x2, y2), (0, 255, 0), 4)
        cv2.putText(frame, predicted_character, (x1, y1 - 10),
                    cv2.FONT_HERSHEY_SIMPLEX, 1.3, (0, 255, 0), 3, cv2.LINE_AA)

    cv2.imshow('Sign Language Detector', frame)

    # Appuyer sur 'q' pour quitter
    if cv2.waitKey(1) & 0xFF == ord('q'):
        break

cap.release()
cv2.destroyAllWindows()
